#!/usr/bin/env Rscript

## you can run this script via
## Rscript --save ./Prov_QDR-RE_clnr.R
## in your command line
## on Unix with R installed this will work in the directory the script is located
## on Windows you have to indicate the program in script paths accordingly

## This script is intended to run larger quantities of provenance data at once

rm(list = ls())
# dev.off(dev.list()["X11cairo"])

##===================================
## Preamble

## set a working directory; tables with your provenance tracer data
## "/home/Arrgh/Provenance"
setwd(file.path(
  "/home",
  "/Arrgh",
  "/Provenance"
  ))

data12 <- "Aare/VAN20002238_1_Lizaga.csv"
stream1 <- "Aare"
## nice to have
## variable to define the conservation samples
conserve1 <- c(3,9)
## variable to define the samples of the core
debther <- c(10:61)
## give the number of core sections
crsec <- seq(1,213)
## core sections that were retrieved in two or more pieces
## indicates the number of pieces in excess
skip <- c(37,39,41,101)

data22 <- "Rhone/Rhone_wrk_Lizaga.csv"
stream2 <- "Rhone"
## variable to define the conservation samples
conserve2 <- c(2,6,13,18,20,22)

## elements given in ppm
ppm <- c("Ba", "Ni", "Sr", "Zr", "Y", "Nb", "Sc")
## oxides given in percent
pct <- c("SiO2", "Al2O3", "Fe2O3", "MgO", "CaO", "Na2O", "K2O", "TiO2",
         "P2O5", "MnO", "Cr2O3")

## remove some data from the set, if you dare fi
xcld <- FALSE
rem <- c("Lue")#,"Bal","Vis")

## only get LDA and PCA for all data
lim.dat <- FALSE

## two plots of the entire data set are always produced
## LDA and PCA
## choose if this display should come from a reduced data set (by stepwise LDA)
## if reduction is chosen, please set the maximum number of variables
## please note, this step is skipped if you hand pick a set of variables below
red.all <- FALSE
maxvar <- 18#max18

## disables the 'red.all' step
## indicate if you have chosen a set of variables to provide a 'stable'
## setup for the analysis
pckd <- FALSE
hnd.pck <-
  # c("SiO2","Al2O3","Fe2O3","MgO","CaO","Na2O","K2O","TiO2","P2O5","MnO","Cr2O3",
  #   "Ba","Ni","Sr","Zr","Y","Nb","Sc")#entire data set
# hnd.2 <-
  c("MnO","TiO2","Fe2O3","Sr","Al2O3","K2O","Y",
    "P2O5","Ni","Na2O","Cr2O3","SiO2","CaO") #b2+5
  # "MnO","TiO2","Fe2O3","Sr","Al2O3","K2O","Y","P2O5" #b2
  # "MnO","TiO2","P2O5","Na2O","Y","K2O","Al2O3","Ni" #b5

## choose if PCA data should be center log-ratio transformed
## or if center and scaling via 'prcomp' function is applied
clr <- TRUE

## on a first run it might better be "FALSE", as it is pretty annoying
plotting <- FALSE#c("LDA","MDS","PCA2")
#c("PCA2","MDSchck")
#c("MDS","MDSchck")
#c("PCA","LDA","MDS","MDSchck","PCA2","cmplt")
# FALSE # TRUE

## export to excel files, all data that is in tables
excel <- TRUE

## your expected endmembers as used in your data
## rearrange order to change order in the final core plot
mmbrs <- c("Helvetic", "External", "Penninic", "sPenninic")

## for plotting with consistent coloring
clrs <- c("Core" = "black"
          ,"Stream" = "black"
          , "External" = "#8c510a" #rgb(140, 81, 10)
          , "Helvetic" = "#d8b365" #rgb(216, 179, 101)
          , "Penninic" = "#5ab4ac" #rgb(90, 180, 172)
          , "sPenninic" = "#01665e") #rgb(1, 102, 94)

## will require user input on every sample, be prepared
## if unmixing is chosen, plotting is automatically activated
## to avoid input of 'enter', run script from command line
## e.g., Unix: got to directory with script and data, type 
## Rscript --save ./Prov_QDR-RE_clnr.R
unmixing <- TRUE#TRUE#FALSE
samples <- 100L
iter <- 100L
seed <- 123456L

## provide a naming scheme for your output
## e.g., date, options chosen
nameing <- '211230_av-ac-100'

##===================================
## Script

options(warn = 0)

## Lizaga's package
library(fingerPro)
## Vermeesch's package
library(provenance)
## Comas' package???
## is not really running the functions wanted for
# library(robCompositions)
## use pipes
library(magrittr)
## use transpositioning
# library(data.table)
## use lda
library(MASS)
## use stepclass (LDA)
library(klaR)
## plot with ggplot
library(ggplot2)
library(ggfortify)
library(ggrepel)
# library(plotly)
# library(orca)
## push to excel
library(openxlsx)

filebase <- getwd()

## this opens an instance to save all the plots that are created
plt.file <- file.path(filebase,paste(nameing, ".pdf",sep = ""))

## this opens an instance to save all the console output
tab.file <- file.path(filebase,paste(nameing, "_output.txt",sep = ""))

cat("Hello my dear friend.","\n",
    "You will find some plots at", plt.file,"\n",
    "and a log at", tab.file,"\n")

## this opens an instance to save all the plots that are created
plt.file <- file.path(filebase,paste(nameing, ".pdf",sep = ""))

## this opens an instance to save all the console output
tab.file <- file.path(filebase,paste(nameing, "_output.txt",sep = ""))
sink(file = tab.file)

cat("We are up and running.")
cat("\n",plt.file,"\n",tab.file)

## the primary files to work with; i.e. data tables
datlocone <- file.path(filebase,data12)
datloctwo <- file.path(filebase,data22)
cat("\n","Data paths set.")

datone <- read.csv(datlocone, header = TRUE, sep = ',')
row.names(datone) <- datone[,"Sample"]
tmp.datone <- datone[,!names(datone) %in% c("Sample","Geo")]
for (pc in names(tmp.datone)){
  if (pc %in% ppm){
    tmp.datone[pc] <- tmp.datone[pc] / 10000
  }
}
tmp.sum <- rowSums(tmp.datone)
for (re in names(tmp.datone)){
  tmp.datone[re] <- tmp.datone[re] / tmp.sum
}
for (pc in names(tmp.datone)){
  if (pc %in% ppm){
    tmp.datone[pc] <- tmp.datone[pc] * 10000
  } else if (pc %in% pct){
    tmp.datone[pc] <- tmp.datone[pc] * 100
  }
}

datone <- cbind(datone[c('Sample','Geo')],tmp.datone)
cat("\n","Data set 1 can be processed.")

dattwo <- read.csv(datloctwo, header = TRUE, sep = ',')
row.names(dattwo) <- dattwo[,"Sample"]
tmp.dattwo <- dattwo[,!names(dattwo) %in% c("Sample","Geo")]
for (pc in names(tmp.dattwo)){
  if (pc %in% ppm){
    tmp.dattwo[pc] <- tmp.dattwo[pc] / 10000
  }
}
tmp.sum <- rowSums(tmp.dattwo)
for (re in names(tmp.dattwo)){
  tmp.dattwo[re] <- tmp.dattwo[re] / tmp.sum
}
for (pc in names(tmp.dattwo)){
  if (pc %in% ppm){
    tmp.dattwo[pc] <- tmp.dattwo[pc] * 10000
  } else if (pc %in% pct){
    tmp.dattwo[pc] <- tmp.dattwo[pc] * 100
  }
}

dattwo <- cbind(dattwo[c('Sample','Geo')],tmp.dattwo)
cat("\n","Data set 2 can be processed.")

rm(tmp.datone,tmp.dattwo,tmp.sum,pc,re)

tmp.cons <- c()
if (grep("Stream",datone$Geo) == conserve1 &&
    grep("Core",datone$Geo) == debther &&
    grep("Stream",dattwo$Geo) == conserve2
) {
  for (c in conserve2) {
    tmp.cons <- c(tmp.cons,c+nrow(datone))
  }
  conservatio <- c(conserve1,tmp.cons);rm(tmp.cons)
} else {stop("There is a missmatch between the order of samples that you
provided in the data and the order indicated in the preamble")}

rm(c)

##===================================
## exclude sth, if you dare fi 
##-----------------------------------
if(xcld && length(rem)){
  for(ck in rem){
    if(ck %in% datone$Sample){
      datone <- datone[-grep(ck,datone$Sample),]}
    if(ck %in% dattwo$Sample){
      dattwo <- dattwo[-grep(ck,dattwo$Sample),]}
  }
} else if(xcld && !length(rem)){
  cat("You wanted exclusion, but none was provided.")
} else cat("No entries excluded.")
rm(ck)


##===================================
## Data prep
##-----------------------------------
## Amalgamate the data sets and exclude the stream and source samples
## in preparation for Conservation (?), H-Test, LDA (?)
all.dat <- rbind(datone,dattwo)

## This extraction is slightly more elegant, than going by numbers
tmp.datone <- datone[-grep("Stream",datone$Geo),]
tmp.dattwo <- dattwo[-grep("Stream",dattwo$Geo),]
# source.dat <- rbind(tmp.datone,tmp.dattwo)
core.dat <- rbind(tmp.dattwo,tmp.datone); rm(tmp.datone,tmp.dattwo)

tmp.datone <- datone[-grep("Core",datone$Geo),]
# tmp.dattwo <- dattwo[grep("Stream",dattwo$Geo),]
# stream.dat <- rbind(tmp.datone,tmp.dattwo); rm(tmp.datone,tmp.dattwo)
catch.bundl <- list(tmp.datone, dattwo); rm(tmp.datone)
strm.nms <- c(stream1,stream2)
names(catch.bundl) <- strm.nms; rm(strm.nms)

##===================================
## Data frame prep
##-----------------------------------
## a little tinkering
## to create tables of the stream samples and
## to create tables that contain each of the 61 readings from the core
## and for them to be analyzed
lst.spct <- list()

## to aggregate each down-stream (ds) sample and up-stream catchments
## in individual dfs, all collected in one list
tmp.spct.nms <- list()
ctr <- 0
for(m in catch.bundl){
  prov.ds.lst <- list()
  prov.ds.nms <- list()
  ctr <- ctr + 1
  # tmp.dat <- m
  tmp.name <- paste("prov.",names(catch.bundl[ctr]),".lst",sep = "")
  for(n in grep("Stream",m$Geo)){
    ## has to be refreshed every iteration
    tmp.dat <- m
    namer <- row.names(tmp.dat[n,])
    cat("\n","\n","This is an iteration on sample",namer,
        "of cutting the data.")
    tmp.strm <- grep("Stream",tmp.dat$Geo)
    tmp.strm <- tmp.strm[tmp.strm != n]
    if(n < nrow(tmp.dat)){
      tmp.src <- seq(n+1,nrow(tmp.dat))
      tmp.strm <- c(tmp.strm,tmp.src)
    }
    tmp.dat <- tmp.dat[-tmp.strm,]
    prov.ds.lst <- c(prov.ds.lst, list(tmp.dat))
    prov.ds.nms[[length(prov.ds.nms)+1]] <- namer
    cat("\n","The cropped data is available in",tmp.name,"/$",namer)
  }
  tmp.spct.nms <- c(tmp.spct.nms, list(tmp.name))
  names(prov.ds.lst) <- prov.ds.nms
  assign(tmp.name, prov.ds.lst)
  lst.spct <- c(lst.spct, list(prov.ds.lst))
}
rm(ctr
   ,m
   ,tmp.dat
   ,tmp.name
   ,n
   ,namer
   ,tmp.strm
   ,tmp.src
   ,catch.bundl
   ,prov.ds.lst
   ,prov.ds.nms
)

## to aggregate each down-hole (dh) sample and up-stream catchments
## in individual dfs, all collected in one list
prov.dh.lst <- list()
prov.dh.nms <- list()

for(o in grep("Core",core.dat$Geo)){
  namer <- row.names(core.dat[o,])
  cat("\n","\n","This is an iteration on sample",namer,
      "of cutting the data.")
  tmp.core <- grep("Core",core.dat$Geo)
  tmp.core <- tmp.core[tmp.core != o]
  tmp.dat <- core.dat[-tmp.core,]
  prov.dh.lst <- c(prov.dh.lst, list(tmp.dat))
  prov.dh.nms[[length(prov.dh.nms)+1]] <- namer
  cat("\n","The cropped data is available in prov.dh.lst /$",namer)
}
tmp.spct.nms <- c(tmp.spct.nms, list("prov.dh.lst"))
names(prov.dh.lst) <- prov.dh.nms
lst.spct <- c(lst.spct, list(prov.dh.lst))
names(lst.spct) <- tmp.spct.nms

rm(o
   ,namer
   ,tmp.core
   ,tmp.dat
   ,prov.dh.nms
   ,list = as.character(tmp.spct.nms)
   ,tmp.spct.nms
)

##-----------------------------------
## create a list of plots to print; previously in plotting section
rec.plot.init <- list()

##===================================
## stepwise LDA, reduction of all data 
##-----------------------------------
if(red.all && !pckd){
  stpw.all.dat <- all.dat[-grep("Stream|Core",all.dat$Geo),
                          -!names(all.dat) %in% "Sample"]
  
  ## for stepclass to work, there have to be at least two entries to each
  ## group of endmembers; otherwise it fails; e.g., Rh11 with single Helvetic
  stpw.all.dat <- stepclass(Geo~., stpw.all.dat,
                            method = "lda",
                            criterion = "AS",
                            maxvar = maxvar,
                            direction = "forward")
  red.all.dat <- all.dat[,names(all.dat) %in%
                           c("Geo", "Sample", stpw.all.dat$model$name)] %>%
    as.data.frame()
}


##===================================
## LDA on complete data
##-----------------------------------
if(red.all && !pckd){
  tmp.dat <- red.all.dat
} else if(pckd){
  tmp.dat <- all.dat[,names(all.dat) %in% c("Geo","Sample",hnd.pck)] %>%
    as.data.frame()
} else {
  tmp.dat <- all.dat
  }

all.dat.lda <- lda(Geo~.,
                   data=tmp.dat[-grep("Stream|Core",tmp.dat$Geo),
                                #the Sample column can not be used
                                !names(tmp.dat) %in% "Sample"]
                   )

all.dat.prd <- predict(all.dat.lda,
                       tmp.dat[-grep("Stream|Core",tmp.dat$Geo),
                               !names(tmp.dat) %in% "Sample"])
core.dat.prd <- predict(all.dat.lda,
                        tmp.dat[grep("Core",tmp.dat$Geo),
                                !names(tmp.dat) %in% "Sample"])

## to plot the LDA data, another df has to be aggregated
## this df has to carry two columns with 'LD1' and 'LD2'
all.dat.LD <- rbind(
  cbind(tmp.dat[-grep("Stream|Core",tmp.dat$Geo),],all.dat.prd$x),
  cbind(tmp.dat[grep("Core",tmp.dat$Geo),],core.dat.prd$x)
  )

## use for plot not to crash, ggplot currently crashes (R 4.1.0)
x11()
## VAa2.1 = Aa2 and Rhone-1=R-1, are super-dooper outliers
# tmp.dat <- all.dat.LD#[-grep("Aa2|R-1(?![0-9])",
                            # all.dat.LD$Sample,
                            # perl = TRUE),]
LD.plt <- ggplot(all.dat.LD
                 ,aes(LD1,
                      LD2,
                      colour = Geo)) +
  geom_point() +
  scale_color_manual(values = clrs) +
  geom_text_repel(data = all.dat.LD[grep("Helvetic|External|Penninic|sPenninic",
                                   #Helvetic|External|Penninic|sPenninic
                                   #Core
                                   all.dat.LD$Geo),]
                  , aes(LD1,LD2, label = Sample)
                  , show.legend = FALSE
                  # , vjust = "inward", hjust = "inward"
                  , check_overlap = TRUE) +
  ggtitle("Linear Discrimination of all samples")
# rm(tmp.dat)
print(LD.plt)
rec.plot.init[[length(rec.plot.init)+1]] <- recordPlot()
graphics.off()
cat("\n","Recording Linear Discrimination")

##===================================
## PCA on complete data
##-----------------------------------

if(clr){
  all.dat.pca <- CLR(tmp.dat[-grep("Stream|Core",tmp.dat$Geo),
                         !names(tmp.dat) %in% c("Sample","Geo")]) %>%
    prcomp()
} else {
  all.dat.pca <- prcomp(tmp.dat[-grep("Stream|Core",tmp.dat$Geo),
                            !names(tmp.dat) %in% c("Sample","Geo")],
                    center = TRUE, scale. = TRUE)
}

sum.all.dat.pca <- summary(all.dat.pca)

cat("\n", "Plotting Biplot for all samples")
tmp.plt <- autoplot(all.dat.pca, data = tmp.dat[-grep("Stream|Core",tmp.dat$Geo),],
                    colour = 'Geo',
                    loadings = TRUE, loadings.colour = 'black',
                    loadings.label = TRUE,
                    loadings.label.colour = 'black',
                    loadings.label.vjust = -2,
                    frame = TRUE
                    # , frame.type = 'norm', frame.level = 0.95
                    ) +
  ggtitle(paste("Biplot of all samples")) +
  scale_color_manual(values = clrs) #+
  # geom_text(data = tmp.pca$x[!rownames(tmp.pca$x) %in% c("Stream","Core"),]
  #           , show.legend = FALSE
  #           , vjust = "inward", hjust = "inward"
  #           , check_overlap = TRUE
  #           # vjust=-1,
  #           , label = tmp.pca$x[!rownames(tmp.pca$x) %in% c("Stream","Core"),]
  #           # , check_overlap = TRUE)
  #           )

## on R 4.1.0 a bug prohibits plotting with gg in the std graphics
## call for "X11()" required
x11()
print(tmp.plt)
rec.plot.init[[length(rec.plot.init)+1]] <- recordPlot()
graphics.off()
##-----------------------------------

if(!lim.dat){
  ##===================================
  ## Treatment
  ##-----------------------------------
  ## list of primary PCA results
  dat.pca <- list()
  dat.pca.nms <- list()
  if(!pckd){
    ## list of conservation candidates
    dat.cons <- list()
    dat.cons.nms <- list()
    ## list after application of conservation
    lst.pcons <- list()
    lst.pcons.nms <- list()
    ## list of K-W test results
    dat.KW <- list()
    dat.KW.nms <- list()
    ## condensed list of K-W test results
    lst.KW <- list()
    ## list after K-W test
    lst.pKW <- list()
    lst.pKW.nms <- list()
    ## resulst of stepwise LDA
    dat.stpwLDA <- list()
    dat.stpwLDA.nms <- list()
    ## list of reduced lda results
    lst.stpwLDA <- list()
    lst.stpwLDA.nms <- list()
  }
  ## list after LDA
  lst.LDA <- list()
  lst.LDA.nms <- list()
  ## list of prediction outcomes LDA
  dat.vis <- list()
  dat.vis.nms <- list()
  ## list of secondary PCA results
  dat.pca2 <- list()
  dat.pca2.nms <- list()
  ## list of MDS results
  dat.MDS <- list()
  dat.MDS.nms <- list()
  
  if(!pckd){
    ## all in one
    data.lst <- c(
      "dat.cons"## tables
      # ,"dat.KW"## list of lists
      # ,"dat.MDS"## list of lists
      # ,"dat.pca"## list of lists
      # ,"dat.pca2"## list of lists
      # ,"dat.vis"## list of factors
      # ,"dat.unmix"## list of tables
      # ,"lst.LDA"## list of lists
      ,"lst.pcons"## tables
      ,"lst.pKW"## tables
      ,"lst.KW"## tables
      ,"lst.stpwLDA"
    )
  } else data.lst <- c()
  ## umixing results
  if(unmixing){
    require(data.table)
    dat.unmix <- list()
    dat.unmix.nms <- list()
    data.lst <- append(data.lst, "dat.unmix")
    if(!exists("stat.umx") && exists("mmbrs")){
      # mmbrs <- append("GOF", mmbrs)
      stat.umx <- data.table(matrix(NA, nrow = 0, ncol = 0))
      # stat.umx <- rbind(stat.umx, tmp.means, fill = TRUE)
      data.lst <- append(data.lst, "stat.umx")
    } else if(!exists("mmbrs") && length(mmbrs) == 0){
      stop("\n", "We are missing your endmembers variable")
    }
  }
  
  ##-----------------------------------
  ## loop into first level of the list
  ## start a "cntr" to address same level in the list
  ## carry temporary list of plots
  for(l in lst.spct){
    rec.plot.tmp <- list()
    cntr <- 0
    ## advance into second level of the list
    ## let the "cntr" run with th eloop
    for(m in l){
      cntr <- cntr + 1
      ## get the name of current level
      tmp.name <- as.name(names(l[cntr]))
      
      if(pckd && nrow(m) < 6){
        cat("\n",tmp.name, "skipped due to too few samples.")
        next
      }
      
      ##===================================
      ## PCA on data set
      ##----------------------------------
      
      ## fix data in temporary to alter it
      if(pckd){
        tmp.dat <- m[,names(all.dat) %in% c("Geo","Sample",hnd.pck)] %>%
          as.data.frame()
      } else {
        tmp.dat <- m
      }
      ## add the process name
      tmp.stp <- paste("PCA.",tmp.name, sep = "")
      ## check if PCA can really be run on all columns
      ## PCA cannot be run if "sd == 0"
      tmp.comp <- lapply(tmp.dat,sd)
      ## cheap clause to detect unwanted columns
      if(any(tmp.comp == 0, na.rm = TRUE)){
        ## get malicious entry
        ## by name
        tmp.ntr.nm <- names(tmp.comp)[which(tmp.comp==0)]
        cat("\n","\n",tmp.ntr.nm, "could not be used for PCA")
        ## cut it from the data
        tmp.dat <- tmp.dat[,-which(tmp.comp==0)]
      } else {cat("\n","\n","All data to PCA")}
      
      if(clr){
        tmp.pca <- CLR(tmp.dat[,!names(tmp.dat) %in% c("Sample","Geo")]) %>%
          # dist(method = "euclidean") %>%
          prcomp()
      } else {
        tmp.pca <- prcomp(tmp.dat[,!names(tmp.dat) %in% c("Sample","Geo")],
                          center = TRUE, scale. = TRUE)
      }
      dat.pca <- c(dat.pca, list(tmp.pca))
      dat.pca.nms <- c(dat.pca.nms, as.name(tmp.stp))
      
      if(plotting == TRUE || "PCA" %in% plotting){
        cat("\n", "Plotting Biplot for", tmp.stp)
        tmp.plt <- autoplot(tmp.pca, data = tmp.dat, colour = 'Geo',
                            loadings = TRUE, loadings.colour = 'black',
                            loadings.label = FALSE,
                            loadings.label.colour = 'black',
                            loadings.label.vjust = -1,
                            frame = TRUE, frame.type = 'norm') +
          ggtitle(paste("Biplot", tmp.stp)) +
          scale_color_manual(values = clrs) +
          geom_text(vjust=-1, label=rownames(tmp.dat), check_overlap = TRUE)
  
      ## on R 4.1.0 a bug prohibits plotting with gg in the std graphics
      ## call for "X11()" required
        x11()
        print(tmp.plt)
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        graphics.off()
        } else {cat("\n","No PCA plot generated")}
      
      if(!pckd){
        ##===================================
        ## Conservation determination
        ##-----------------------------------
        ## determine conservation x + ... + n = n*a +- 0.2*(n*a)
        ## x = sample 1
        ## n = sample n
        ## a = observed mix
        # cat("\n","\n", "Here we try to evaluate conservation of the observed elements.",
        #     "\n", "First the conservation for the individual stream samples,",
        #     "provided in the 'conserveX' variables, are checked.",
        #     "\n", "Conservation is for only the ultimate stream sample is than used",
        #     "to eliminate measurands.",
        #     "\n", "The rule of thumb used:",
        #     "\n", "x + ... + n = n*a +- 0.2*(n*a)",
        #     "\n", "x = sample 1", "\n", "n = sample n", "\n", "a = observed mix")
      
      
      
        ## add the process name
        tmp.stp <- paste("cons.",tmp.name, sep = "")
        
        cat("\n", "Determining conservation for", tmp.name)
        
        tmp.stream.sum <- lapply(m[-which(m$Geo %in% c("Core","Stream")),],
                               function(x)
                                 if(is.numeric(x)) {sum(x)}
                               ## first two entries can not be empty (=="NULL")
                                 else {as.character(paste("Sum"))}
                               )
        ## The result has to be transposed, WRONG
        ## If the first two entries are 'character' then all is good
        tmp.stream.sum <- as.data.frame(tmp.stream.sum)
        row.names(tmp.stream.sum) <- tmp.stream.sum[,"Sample"]
        
        ## This is ill! Not anymore
        ## the function introduces two new rows with calculations of the
        ## upper and lower boundaries of conservation
        tmp.cnt <- nrow(m[-which(m$Geo %in% c("Core","Stream")),])
        f1 <- function(y){list(
          if(is.numeric(y)){y*tmp.cnt + y*tmp.cnt*0.2}
          else {as.character(paste("UpperBdr"))},
          if(is.numeric(y)){y*tmp.cnt - y*tmp.cnt*0.2}
          else {as.character(paste("LowerBdr"))}
        )}
          
        tmp.source.chck <- sapply(m[which(m$Geo %in% c("Core","Stream")),], f1) %>% 
          as.data.frame()
        ## the classes of the individual columns are not transposed
        ## hence, classes are copied from '.sum' to '.check'
        tmp.source.chck[] <- mapply(FUN = as,tmp.source.chck,
                                    sapply(tmp.stream.sum,class),SIMPLIFY = FALSE)
          
        row.names(tmp.source.chck) <- tmp.source.chck[,"Sample"]
          
        ## Will not work if above is not fixed; is fixed
        ## aggregates the two separate df into one to allow the conservation check
        tmp.cequ <- rbind(tmp.stream.sum,tmp.source.chck)
          rm(tmp.stream.sum,tmp.source.chck)
        row.names(tmp.cequ) <- tmp.cequ[,"Sample"]
        keeper <- as.character(paste("Keeper"))
        ## check if '.sum' is within the '.chck' boundaries
        ## 'Keeper' -> bool
        for(a in seq_along(tmp.cequ)){
          if(
            is.numeric(tmp.cequ[1,a]) &&
            tmp.cequ[1,a] <= tmp.cequ[2,a] &&
            tmp.cequ[1,a] >= tmp.cequ[3,a]){tmp.cequ[keeper,a] <- TRUE}
          else tmp.cequ[keeper,a] <- FALSE
          # if(is.numeric(tmp.cequ[1,a]) && 
          #    tmp.cequ[1,a] > tmp.cequ[2,a]){
          #   tmp.cequ["Diff",a] <- tmp.cequ[1,a] - tmp.cequ[2,a]
          # }else if(is.numeric(tmp.cequ[1,a]) && 
          #          tmp.cequ[1,a] < tmp.cequ[3,a]){
          #   tmp.cequ["Diff",a] <- tmp.cequ[1,a] - tmp.cequ[3,a]
          # }else tmp.cequ["Diff",a] <- TRUE
        }
        
        tmp.lst <- assign(tmp.stp, tmp.cequ)
        dat.cons <- c(dat.cons, list(tmp.lst))
        dat.cons.nms <- c(dat.cons.nms, as.name(tmp.stp))
        rm(tmp.lst, list = tmp.stp)
        
        
        ##===================================
        ## Conservation application
        ##-----------------------------------
        cat("\n", "Now we make use of the conservation criterion for", tmp.name)
        
        tmp.dat <- m
        tmp.stp <- paste("pcons.",tmp.name, sep = "")
        ## braucht es glaub nicht die Ebene
        # for(z in g) {
        # counter <- counter + 1
        # tmp.dat <- as.data.frame(lst.spct[[run]][[counter]])
        # namer <- names(lst.spct[[run]][counter])
        cat("\n", "The data of sample", tmp.name, "is reduced")
        # colnames(tmp.dat) <- colnames(datone)
        cl.nms <- colnames(tmp.dat)
        clms <- c()
        tmp.pcons <- data.frame(matrix(NA, nrow = nrow(tmp.dat), ncol = 0))
        cat("\n", "based on the following conservation", "\n")
        print(tmp.cequ[nrow(tmp.cequ),])
        for(w in seq_along(tmp.cequ[nrow(tmp.cequ),])){
          if(is.numeric(tmp.cequ[nrow(tmp.cequ),w]) &&
             tmp.cequ[nrow(tmp.cequ),w] == 1){
            tmp.pcons <- cbind(tmp.pcons,tmp.dat[,w])
            clms <- c(clms,cl.nms[w])
          } else if(is.character(tmp.cequ[nrow(tmp.cequ),w]) &&
                    tmp.cequ[nrow(tmp.cequ),w] == FALSE){
            tmp.pcons <- cbind(tmp.pcons,tmp.dat[,w])
            clms <- c(clms,cl.nms[w])
          } else {cat("\n", "Column", colnames(tmp.cequ)[w], "was omitted.")}
        }
        colnames(tmp.pcons) <- clms; rm(clms)
        rownames(tmp.pcons) <- tmp.pcons[,"Sample"]
        tmp.lst <- assign(tmp.stp, tmp.pcons)
        lst.pcons <- c(lst.pcons, list(tmp.lst))
        lst.pcons.nms <- c(lst.pcons.nms, as.name(tmp.stp))
        rm(tmp.lst, list = tmp.stp)
        
        ##===================================
        ## Kruskal-Wallis
        ##-----------------------------------
        cat("\n", "\n", "Performing Kruskal-Wallis H-Test")
        tmp.stp <- paste("KW.",tmp.name, sep = "")
        tmp.KW <- lapply(tmp.pcons, function(x)
          if(is.numeric(x))
            kruskal.test(x~tmp.pcons[,"Geo"])
          else {as.character(paste("placeholder"))}
        )
        tmp.KW[c("Geo","Sample")] <- tmp.pcons[nrow(tmp.pcons),c("Geo","Sample")]
        tmp.lst <- assign(tmp.stp, tmp.KW)
        dat.KW <- c(dat.KW, list(tmp.lst))
        dat.KW.nms <- c(dat.KW.nms, as.name(tmp.stp))
        rm(tmp.lst, list = tmp.stp)
    
        ## another round to get the chi-squared and p-values
        tmp.df <- data.frame(matrix(nrow = 0, ncol = 3))
        colnames(tmp.df) <- c("Element","chi.squared","p.value")
        tmp.nms <- list()
        for(e in seq_along(tmp.KW)){
          if("htest" != class(tmp.KW[[e]])){next}
          else {
            tmp.nm <- names(tmp.KW[e])
            tmp.lst <- data.frame(
              "Element" = tmp.nm,
              "chi.squared" =
                as.numeric(tmp.KW[[tmp.nm]][["statistic"]]),
              "p.value" =
                as.numeric(tmp.KW[[tmp.nm]][["p.value"]])
                            )
            tmp.df <- rbind(tmp.df,tmp.lst)
            tmp.nms <- c(tmp.nms, as.name(tmp.nm))
          }
        }
        rownames(tmp.df) <- tmp.nms; rm(tmp.nms,tmp.lst,tmp.nm)
        
        lst.KW <- c(lst.KW, list(tmp.df))
        
        cat("\n", "Kruskal-Wallis performed on data", tmp.name)
        
        ##===================================
        ## K-W used
        ##-----------------------------------
        cat("\n","\n", "Now we make use of the p-values to exclude further data")
        tmp.stp <- paste("pKW.",tmp.name, sep = "")
        
        cat("\n", "The data in", tmp.name, "is reduced")
        # colnames(tmp.dat) <- colnames(pcon.lst[[run]][[counter]])
        cl.nms <- colnames(tmp.pcons)
        clms <- c()
        tmp.pKW <- data.frame(matrix(NA, nrow = nrow(tmp.pcons), ncol = 0))
        cat("\n", "based on the following p-values")
        if(nrow(tmp.pcons) <= 2) next
        for(w in seq(tmp.KW)){
          # ctr <- ctr + 1
          if(length(tmp.KW) == length(tmp.pcons)){
            # cat("\n", "There is data to check.")
            if(is.character(tmp.KW[[w]])){
              # cat("\n", "The data was of type 'character'")
              tmp.pKW <- cbind(tmp.pKW,tmp.pcons[,w])
              clms <- c(clms,cl.nms[w])
            } else if(is.numeric(tmp.KW[[w]][["p.value"]]) &&
                      tmp.KW[[w]][["p.value"]] <= 0.05){
              # cat("\n", "The data was 'numeric' and matched the criterion")
              tmp.pKW <- cbind(tmp.pKW,tmp.pcons[,w])
              clms <- c(clms,cl.nms[w])
              cat("\n", names(tmp.KW)[w], tmp.KW[[w]][["p.value"]])
            } else {cat("\n", names(tmp.KW)[w], tmp.KW[[w]][["p.value"]]
                        , "This column should be omitted.")}
          ## if "KW.dat" is empty on the lowest level, there is a mismatch
          ## and the entry can be skipped
          } else {cat("\n", "The KW test can not be matched with the conservation results")}
          # cat("\n", "Column", w, "checked")
        }
        
        colnames(tmp.pKW) <- clms; rm(clms)
        rownames(tmp.pKW) <- tmp.pKW[,"Sample"]
        
        ##-----------------------------------
        ## to check what all other elements would have yielded
        ## and use if entries are missing right on
        tmp.dat <- m
        tmp.df <- tmp.dat[,c("Geo","Sample")]
        tmp.dat <- tmp.dat[,-which(colnames(tmp.dat) %in%
                                     colnames(tmp.pKW))]
        tmp.dat <- cbind(tmp.df,tmp.dat); rm(tmp.df)
        xtr.KW <- lapply(tmp.dat, function(x)
          if(is.numeric(x))
            kruskal.test(x~tmp.dat[,"Geo"])
          else {as.character(paste("placeholder"))}
        )
        xtr.KW[c("Geo","Sample")] <- tmp.dat[nrow(tmp.dat),c("Geo","Sample")]
    
        ## see the results of the extra K-W test
        ## sort them from best to worst results
        ## use sorting later on if necessary
        xtr.rule <- data.frame(matrix(data= NA, nrow=0, ncol = 2))
        colnames(xtr.rule) <- c("Element","chi")
        for(b in names(xtr.KW[-c(1,2)])){
          tmp.df <- data.frame("Element" = b,"chi" = xtr.KW[[b]][["statistic"]])
          xtr.rule <- rbind(xtr.rule,tmp.df)}
        xtr.rule <- xtr.rule[order(-xtr.rule$chi),];rm(xtr.KW)
    
        ## if there are too few entries, call the best from those that
        ## left due to the conservation rule
        ## problematic if ncol == 1; solution: force 'as.data.frame'
        if(ncol(as.data.frame(tmp.pKW[,-c(1,2)])) < 4){
          no <- 0
          while(no + ncol(as.data.frame(tmp.pKW[,-c(1,2)])) < 4){no <- no + 1}
          ## get from the sorted list of entries the best, which are on top
          ## according to no. of missing entries
          tmp.elm <- c(xtr.rule$Element[seq(1,no)]);rm(no)
          tmp.df <- as.data.frame(m[,tmp.elm])
          colnames(tmp.df) <- tmp.elm
          tmp.pKW <- cbind(tmp.pKW,tmp.df)
          cat("\n", "Entries", tmp.elm, "were added back",
          "to attempt further calculations")
        }
        rm(xtr.rule,tmp.elm,tmp.df)
        
        ##-----------------------------------
        
        ## check if there are actually four or more entries to work with
        ## then, check if there are at least three different types of
        ## endmembers in the df; was moved up from next step
        if (ncol(tmp.pKW) >= 4 &&
            ## ">3" from "Stream|Core" plus two endmembers is less than required
            length(unique.default(sapply(tmp.pKW[,"Geo"],unique))) > 3){
          tmp.lst <- assign(tmp.stp ,tmp.pKW)
          lst.pKW <- c(lst.pKW, list(tmp.lst))
          lst.pKW.nms <- c(lst.pKW.nms, list(tmp.stp))
          rm(tmp.lst,list = tmp.stp)
          } else {
          cat("\n", tmp.stp, "was reduced to no data.",
              "\n", "Not enough endmembers were recognized.")
          rm(tmp.pKW,tmp.stp)
          }
    
        ##===================================
        ## stepwise Linear Discrimination
        ##-----------------------------------
        ## if "tmp.pKW" was deleted right before, go 'next'
        if(!exists("tmp.pKW")) next
        
        cat("\n", "\n", "Attempting stepwise Linear Discrimination Analysis")
        tmp.stp <- paste0("stpwLDA.",tmp.name)
        
        tmp <- tmp.pKW
        
        if(ncol(tmp) - 2 <= 4){
          cat("\t", "Four variables or less.")
        } else if(any(table(tmp$Geo[-grep("Stream|Core",tmp$Geo)]) == 1)){
          cat("\t", "One endmember with only one observation.")
        } else {
          # print(ncol(tmp[,-which(colnames(tmp) %in% "Geo")]))
          cat("\t","Should be displayed if there are more than four variables",
              "and enough observations of each endmember.", "\n")
          tmp <- tmp[-grep("Stream|Core",tmp$Geo),
                             -!names(tmp) %in% "Sample"]
          ## for stepclass to work, there have to be at least two entries to each
          ## group of endmembers; otherwise it fails; e.g., Rh11 with single Helvetic
          tmp.stpwlda <- stepclass(Geo~., tmp,
                                   method = "lda",
                                   criterion = "AS",
                                   maxvar = 4,
                                   direction = "forward")
          
          dat.stpwLDA <- c(dat.stpwLDA, list(tmp.stpwlda$process))
          dat.stpwLDA.nms <- c(dat.stpwLDA.nms, as.name(tmp.stp))
          
          ## funzt ned! so geht's
          tmp.psLDA <- tmp.pKW[,names(tmp.pKW) %in%
                                 c("Geo", "Sample", tmp.stpwlda$model$name)] %>%
            as.data.frame()
          
          lst.stpwLDA <- c(lst.stpwLDA, list(tmp.psLDA))
          lst.stpwLDA.nms <- c(lst.stpwLDA.nms, as.name(tmp.stp))
        }
      }
      
      ##-----------------------------------
      ## MOVED up
      ## then, check if there are at least three different groups of entries in the df
      # if(length(unique.default(sapply(tmp.pKW[,"Geo"],unique))) > 2){
      #   ## and perform the LDA
      #   tmp.LDA <- lda(Geo~., data=tmp.pKW[-grep("Stream|Core",tmp.pKW$Geo),
      #                                      ##the Sample column can not be used
      #                                      !names(tmp.pKW) %in% "Sample"])
      #   cat("\n", "LDA performed on data", tmp.name)
      #   ## otherwise, calculating an LDA is nonsense
      # } else {tmp.LDA <- list(tmp.pKW[,"Geo"])} ## WTF?
      ##===================================
      ## LDA on the reduced data
      ##-----------------------------------
      
      if(pckd){
        tmp.dat <- m[,names(all.dat) %in% c("Geo","Sample",hnd.pck)] %>%
          as.data.frame()
        tmp.stp <- paste0("hp_LDA.",tmp.name)
        cat("\n", "LDA performed on hand picked data of", tmp.name)
      } else if(exists("tmp.psLDA")){
        tmp.dat <- tmp.psLDA
        cat("\n", "LDA performed on stepwise data of", tmp.name)
      } else if(!exists("tmp.psLDA")){
        tmp.dat <- tmp.pKW
        cat("\n", "LDA performed on H-Test data of", tmp.name)
      }
      
      # corr.m <- cor(tmp.dat[-grep("Stream|Core",tmp.dat$Geo),
      #                       ##the Sample column can not be used
      #                       !names(tmp.dat) %in% c("Geo","Sample")])
      # if(!any(corr.m == -1) || !any(corr.m == 1)){
        tmp.LDA <- lda(Geo~., data=tmp.dat[-grep("Stream|Core",tmp.dat$Geo),
                                          ##the Sample column can not be used
                                          !names(tmp.dat) %in% "Sample"])
        # cat("\n", "LDA performed on stepwise data of", tmp.name)
        
        ## now, is the "Stream|Core" sample leaning towards any of the groups
        tmp.vis <- predict(tmp.LDA, tmp.dat[grep("Stream|Core",tmp.dat$Geo),
                                              !names(tmp.dat) %in% "Sample"])
        dat.vis <- c(dat.vis, list(tmp.vis$class))
        dat.vis.nms <- c(dat.vis.nms, as.name(tmp.stp))
        # tmp.vis.lst <- c(tmp.vis.lst, list(tmp.vis$class))
        # cat("\n", "Class");print(tmp.vis$class);cat("predicted for", tmp.name)
        tmp.vis <- predict(tmp.LDA, tmp.dat[,!names(tmp.dat) %in% "Sample"])
        compare.lda <- table(tmp.vis$class,tmp.dat$Geo)
        
        cat("\n", "Here is a first view on the linear discrimination:", "\n")
        print(compare.lda)
        
        tmp.lst <- assign(tmp.stp, tmp.LDA)
        lst.LDA <- c(lst.LDA, list(tmp.lst))
        lst.LDA.nms <- c(lst.LDA.nms, as.name(tmp.stp))
        rm(tmp.lst, list = tmp.stp)
        
        ## a quick check, to verify same length
        if(nrow(tmp.dat) == nrow(tmp.vis$x)){
          ## aggregate a list to compare the "Geo" assingment vs prediction
          tmp.vis.cmp <- cbind(tmp.dat[,c("Geo","Sample")],predict = tmp.vis$class)
          tmp.vis.cmp$acc <- ifelse(tmp.vis.cmp$Geo == tmp.vis.cmp$predict,
                                    TRUE, FALSE)
          ## write to file, how prediction went
          for(p in seq(nrow(tmp.vis.cmp))){
            if(tmp.vis.cmp$Geo[p] == tmp.vis.cmp$predict[p]){
              cat("\n","Prediction for");print(rownames(tmp.vis.cmp[p,]));
              cat("correct")
            } else {
              cat("\n","Predicted");print(as.character(tmp.vis.cmp$predict[p]))
              cat("instead of");print(tmp.vis.cmp$Geo[p])
              cat("for");print(rownames(tmp.vis.cmp[p,]))
            }
          }
          
          if(plotting == TRUE || "LDA" %in% plotting){
            ## aggregate a list for the LDA plot
            tmp.LDd <- cbind(tmp.dat[,c("Geo","Sample")],tmp.vis$x)
            ## plot
            LD.plt <- ggplot(tmp.LDd
                             ,aes(LD1,
                                  LD2,
                                  colour = Geo)) +
              geom_point() +
              scale_color_manual(values = clrs) +
              geom_text(data = tmp.LDd#[grep("Stream|Core",tmp.LDd$Geo),]
                        , aes(LD1,LD2, label = Sample)
                        , show.legend = FALSE
                        , vjust = "inward", hjust = "inward"
                        , check_overlap = TRUE) +
              ggtitle("Linear Discrimination of", tmp.name)
          
            x11()
            print(LD.plt)
            rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
            graphics.off()
            cat("\n","Recording Linear Discrimination")
          } else {cat("\n","No LDA plot generated")
            }
        } else {cat("\n","There was a missmatch beetween the samples used for",
                      "LDA, and the results in the prediction based on the LDA")
        }
      # } else {cat("\n","No LDA performed. Data is correlated.")}
      # rm(corr.m)
      
      ##===================================
      ## PCA2 on reduced data set
      ##----------------------------------
      
      if(pckd){
        tmp.dat <- m[,names(all.dat) %in% c("Geo","Sample",hnd.pck)] %>%
          as.data.frame()
        cat("\n", "Using hand picked data.")
        tmp.stp <- paste0("hp_PCA2.",tmp.name)
        } else if(exists("tmp.psLDA")){
          tmp.dat <- tmp.psLDA
          cat("\n", "Using data reduced trough stepwise LDA.")
          tmp.stp <- paste0("stpw_PCA2.",tmp.name)
        } else if(!exists("tmp.psLDA")){
          tmp.dat <- tmp.pKW
          tmp.stp <- paste0("KW_PCA2.",tmp.name)
        }
      ## add the process name
      # tmp.stp <- paste0("PCA2.",tmp.name)
      
      ## check if PCA can really be run on all columns
      ## PCA cannot be run if "sd == 0"
      tmp.comp <- lapply(tmp.dat,sd)
      ## cheap clause to detect unwanted columns
      if(any(tmp.comp == 0, na.rm = TRUE)){
        ## get malicious entry
        ## by name
        tmp.ntr.nm <- names(tmp.comp)[which(tmp.comp==0)]
        cat("\n","\n",tmp.ntr.nm, "could not be used for PCA")
        ## cut it from the data
        tmp.dat <- tmp.dat[,-which(tmp.comp==0)]
      } else {cat("\n","\n","All data to PCA2")}
      
      if(clr){
        tmp.pca <- CLR(tmp.dat[,!names(tmp.dat) %in% c("Sample","Geo")]) %>%
        # dist(method = "euclidean") %>%
        prcomp()
      } else {
        tmp.pca <- prcomp(tmp.dat[,!names(tmp.dat) %in% c("Sample","Geo")],
                          center = TRUE, scale. = TRUE)
      }
      dat.pca2 <- c(dat.pca2, list(tmp.pca))
      dat.pca2.nms <- c(dat.pca2.nms, as.name(tmp.stp))
      
      if(plotting == TRUE || "PCA2" %in% plotting){
        cat("\n", "Plotting Biplot for", tmp.stp)
        tmp.plt <- autoplot(tmp.pca
                            , data = tmp.dat
                            , colour = 'Geo'
                            , loadings = TRUE
                            , loadings.colour = 'black'
                            , loadings.label = TRUE
                            , loadings.label.colour = 'black'
                            , loadings.label.vjust = -1
                            , frame = TRUE
                            # , frame.type = 'norm'
                            ) +
          ggtitle(paste("Biplot", tmp.stp)) +
          scale_color_manual(values = clrs) +
          geom_text(vjust=-1, label=rownames(tmp.dat), check_overlap = TRUE)
      
      ## if some clustering check would be needed, use pam();
      ## rather agnes() or HCPC()
      # pam(lst.pKW$pKW.V072.01[-grep("Core",lst.pKW$pKW.V072.01$Geo),]
      #     , k = 5
      #     , stand = TRUE) +
      #   geom_text(vjust=-1
      #             , label=rownames(lst.pKW$pKW.V072.01
      #                              [-grep("Core",lst.pKW$pKW.V072.01$Geo),])
      #             , check_overlap = TRUE)
        
      ## on R 4.1.0 a bug prohibits plotting with gg in the std graphics
      ## call for "X11()" required
        x11()
        print(tmp.plt)
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        graphics.off()
      } else {cat("\n","No PCA2 plot generated")}
      
      ##===================================
      ## MDS on reduced data
      ##----------------------------------
      
      if(pckd){
        tmp.dat <- m[,names(all.dat) %in% c("Geo","Sample",hnd.pck)] %>%
          as.data.frame()
        cat("\n", "Using hand picked data.")
        tmp.stp <- paste("hp_MDS.",tmp.name, sep = "")
      } else if(exists("tmp.psLDA")){
        tmp.dat <- tmp.psLDA
        cat("\n", "Using data reduced trough stepwise LDA.")
        tmp.stp <- paste("stpw_MDS.",tmp.name, sep = "")
      } else if(!exists("tmp.psLDA")){
        tmp.dat <- tmp.pKW
        tmp.stp <- paste("KW_MDS.",tmp.name, sep = "")
      }
      
      # tmp.stp <- paste("MDS.",tmp.name, sep = "")
      
      tmp.MDS <- cbind(tmp.dat[,names(tmp.dat) %in% c("Sample","Geo")],
                       CLR(tmp.dat[,!names(tmp.dat) %in% c("Sample","Geo")])) %>%
        dist(method = "euclidean") %>%
        isoMDS(k = 2)
      
      # tmp.MDS <- dist(tmp.dat
      #                 , method = "euclidean"
      #                 # , diag = TRUE
      #                 # , upper = TRUE
      #                 # , p = 2
      #                 )
      # 
      # tmp.MDS <-  isoMDS(tmp.dist
      #                    , k = 2
      #                    # , p = 2
      #                    )
      
      dat.MDS <- c(dat.MDS, list(tmp.MDS))
      dat.MDS.nms <- c(dat.MDS.nms, as.name(tmp.stp))
  
      if(plotting == TRUE || "MDS" %in% plotting){
        cat("\n", "Plotting MDS scatter for", tmp.stp)
        tmp.dat <- data.frame(x = tmp.MDS$points[,1],
                              y = tmp.MDS$points[,2],
                              Geo = tmp.dat$Geo)
        tmp.plt <- ggplot(tmp.dat,
                          aes(x = x,
                              y = y,
                              colour = Geo)) +
          xlab("Dim 1") +
          ylab("Dim 2") +
          geom_point() +
          ggtitle(paste("K's non-metric MDS", tmp.name, "for transformed data")) +
          scale_color_manual(values = clrs) +
          geom_text(vjust=-1,
                    label=rownames(tmp.dat),
                    # [,-grep("Core|Stream", tmp.dat$Geo)]),
                    check_overlap = TRUE,
                    show.legend = FALSE)
        x11()
        print(tmp.plt)
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        graphics.off()
        # plot(tmp.MDS$points[,1],
        #      tmp.MDS$points[,2],
        #      xlab = "Dim 1",
        #      ylab = "Dim 2",
        #      main = "Metric MDS",
        #      col = tmp.pKW$Geo)
      } else {cat("\n","No MDS plot generated")}
      
      ##----------------------------------
      if("MDSchck" %in% plotting){
        require(provenance)
        tmp.stp <- paste("MDSchck.",tmp.name, sep = "")
        tmp.dat <- tmp.dat[,-which(names(tmp.dat) %in% c("Geo","Sample"))]
        tmp.dat <- as.compositional(
          tmp.dat,
          method = 'aitchison')
        ## Appply dissimilarity
        tmp.dat.diss <- diss(tmp.dat,method = 'aitchison')
        tmp.MDSnm <- MDS(tmp.dat)
        tmp.MDScl <- MDS(tmp.dat.diss, classical = TRUE)
        cat("\n", "Plotting MDS scatter for", tmp.stp)
        plot(tmp.MDSnm)
        title(sub = paste("Shepard stress", tmp.name))
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        dev.set(which = 2); dev.copy(X11)
        title(main = paste("MDS non-metric", tmp.name))
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        graphics.off()
        plot(tmp.MDScl)
        title(main = paste("MDS metric | classic",tmp.name))
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        cat("\n","Recording MDS plots ", tmp.name, ".", sep = "")
        
        rm(tmp.MDScl,tmp.MDSnm,tmp.dat.diss)
        
      } else {cat("\n","No MDS check plots generated")}

      ##===================================
      ## unmixing
      ##----------------------------------
      cat("\n","\n")
      
      if(unmixing){
        if(pckd){
          tmp.dat <- m[,names(all.dat) %in% c("Geo","Sample",hnd.pck)] %>%
            as.data.frame()
          cat("\n", "Using hand picked data.")
          mp.stp <- paste0("hp_umx.",tmp.name)
        } else if(exists("tmp.psLDA")){
          tmp.dat <- tmp.psLDA
          cat("\n", "Using data reduced trough stepwise LDA.")
          tmp.stp <- paste0("stpw_umx.",tmp.name)
        } else if(!exists("tmp.psLDA")){
          tmp.dat <- tmp.pKW
          tmp.stp <- paste0("KW_umx.",tmp.name)
        }
        
        cat("\n", "Attempting to unmix", tmp.stp)
        
        ## add the process name
        # tmp.stp <- paste("umx.",tmp.name, sep = "")
    
        tmp.dat$Sample <- seq(tmp.dat$Sample)
        tmp.umx <- unmix(tmp.dat, samples = samples, iter = iter, seed = seed)
        
        ## plotting the mix as a barplot
        funck <- function(x) c(mean(x),sd(x))
        
        ## remove the first column
        tmp <- as.data.table(tmp.umx[-1,])
        ## calculate the mean and sd for the end members
        tmp <- tmp[, lapply(.SD, funck), by = id]
        tmp.clnms <- colnames(tmp)
        tmp.means <- tmp[1,-1]
        
        stat.umx <- rbind(stat.umx, tmp.means, fill = TRUE)
        rm(tmp.means)
        
        ## flip/transpose the table
        tmp <- as.data.table(t(tmp))
        colnames(tmp) <- c("mean","sd")
        ## ad a column with the end member names
        tmp$Geo <- tmp.clnms; rm(tmp.clnms)
        tmp <- tmp[-c(1,2),]
        
        tmp.plt <- ggplot(tmp, aes(x=mean, y=Geo, fill = Geo)) +
          geom_bar(stat="identity", color = "black") +
          geom_errorbar(aes(xmin=mean-sd,xmax=mean+sd), width=.5) +
          ggtitle(paste("Umnixing results for", tmp.name)) +
          ylab(NULL) + xlab(NULL) +
          theme(axis.text.y = element_blank(),
                axis.ticks.y = element_blank(),
                axis.text.x = element_text(face = "bold",
                                           size = 16)) +
          scale_fill_manual(values = clrs)
        
        x11()
        print(tmp.plt)
        rec.plot.tmp[[length(rec.plot.tmp)+1]] <- recordPlot()
        graphics.off()
    
        dat.unmix <- c(dat.unmix, list(tmp.umx))
        rm(tmp.umx)
        dat.unmix.nms <- c(dat.unmix.nms, as.name(tmp.stp))
      } else {cat("No umixing performed")}
      ## very important! remove before new iteration
      rm(tmp.psLDA)
    }
    
    rec.plot.init <- append(rec.plot.init,rec.plot.tmp)
    
  }
  
  names(dat.pca) <- dat.pca.nms; rm(dat.pca.nms)
  if(!pckd){
    #
    names(dat.cons) <- dat.cons.nms; rm(dat.cons.nms)
    #
    names(lst.pcons) <- lst.pcons.nms; rm(lst.pcons.nms)
    #
    names(dat.KW) <- dat.KW.nms
    names(lst.KW) <- dat.KW.nms; rm(dat.KW.nms)
    #
    names(lst.pKW) <- lst.pKW.nms; rm(lst.pKW.nms)
    # x.file <- file.path(filebase,paste(nameing, "-lst_pKW.xlsx",sep=""))
    # write.xlsx(lst.pKW,x.file)
    #
    names(dat.stpwLDA) <- dat.stpwLDA.nms; rm(dat.stpwLDA.nms)
    names(lst.stpwLDA) <- lst.stpwLDA.nms; rm(lst.stpwLDA.nms)
  }
  #
  names(lst.LDA) <- lst.LDA.nms; rm(lst.LDA.nms)
  # x.file <- file.path(filebase,paste(nameing, "-lst_LDA.xlsx",sep=""))
  # write.xlsx(lst.LDA,x.file)
  #
  names(dat.vis) <- dat.vis.nms; rm(dat.vis.nms)
  dat.vis <- as.data.frame(dat.vis)
  #
  names(dat.pca2) <- dat.pca2.nms; rm(dat.pca2.nms)
  #
  names(dat.MDS) <- dat.MDS.nms; rm(dat.MDS.nms)
  #
  
  ##===================================
  ## final core plot; only if unmixed
  ##----------------------------------
  if(unmixing){
    names(dat.unmix) <- dat.unmix.nms
    stat.umx <- as.data.frame(stat.umx)
    rownames(stat.umx) <- dat.unmix.nms; rm(dat.unmix.nms)
    stat.umx$names <- rownames(stat.umx)
    
    stat.umx_xt <- as.data.frame(matrix(nrow = 0, ncol = ncol(stat.umx)))
    colnames(stat.umx_xt) <- colnames(stat.umx)
    
    data.lst <- append(data.lst, "stat.umx_xt")
    
    ## create a table that will contain all core sections, except those in "skip"
    ## there will be many empty entries, see exported excel
    for(o in crsec){
      o.str <- as.character(o)
      if(nchar(o.str) == 1){
        o.str <- paste0("00",o)
      } else if(nchar(o.str) == 2){
        o.str <- paste0("0",o)
      }
      if(any(grepl(o.str, stat.umx$names)) == FALSE){
        stat.umx_xt[o,] <- rep(NA, ncol(stat.umx_xt))
        stat.umx_xt[o,]$names <- paste0("umx.N",o.str)
      } else {
        stat.umx_xt <- rbind(stat.umx_xt,
                             stat.umx[grep(o.str,stat.umx$names),])
      }
    }
  
    rrdr <- c("GOF", mmbrs, "names")
    ## re-order the data columns
    stat.umx_xt <- stat.umx_xt[rrdr]
    
    # ## cumulation only depends on length of member list
    # for(m in rev(seq_along(mmbrs))){
    #   ## add 1 for GOF entry
    #   m_in <- m + 1
    #   m_stp <- m + 1
    #   ## cumulate the mixing results for stacked line plots
    #   ## e.g., last entry is all previous entries added to it
    #   while(m_stp > 2){
    #     m_stp <- m_stp - 1
    #     stat.umx_xt[,m_in] <- stat.umx_xt[,m_in] + stat.umx_xt[,m_stp]
    #   }
    # }
    
    stat.umx_xt$count <- seq(1,nrow(stat.umx_xt))  
    stat.umx_xt <- stat.umx_xt[-skip,]
    stat.umx_xt <- as.data.table(stat.umx_xt)
    
    ##----------------------------------
    ## plotly works, orca is strange to me
    # for(p in rev(seq_along(mmbrs))){
    #   p <- p + 1
    #   if(exists("core.plt") == TRUE){
    #     core.plt <- core.plt %>% add_trace(x = stat.umx_xt[,p],
    #                                      y = stat.umx_xt$count,
    #                                      name = names(stat.umx_xt)[p],
    #                                      fill = 'tozerox')
    #   } else if(exists("core.plt") == FALSE){
    #     core.plt <- plot_ly(x = stat.umx_xt[,p],
    #                        y = stat.umx_xt$count,
    #                        type = 'scatter',
    #                        mode = 'lines',
    #                        name = names(stat.umx_xt)[p],
    #                        fill = 'tozerox')
    #   }
    # }
    # core.plt <- core.plt %>% layout(yaxis = list(title = 'Depth',
    #                                            ## attention, the axis is reversed
    #                                            autorange = 'reversed'),
    #                               xaxis = list(title = 'Mixture [%]'))
    # 
    # # x11()
    # # print(core.plt)
    # coreplt.file <- file.path(filebase,paste(nameing, "_coreplt.pdf",sep = ""))
    # orca(core.plt, coreplt.file)
    
    ##----------------------------------
    ## let's use ggplot
    
    # tmp.dat <- stat.umx_xt[,c("External","count")]
    tmp.dat <- melt(stat.umx_xt, id.vars = "count", measure.vars = mmbrs)
    core.plt <- ggplot(data = tmp.dat) +
      geom_area(aes(x = value,
                    y = count,
                    group = variable,
                    # colour = variable,
                    fill = variable),
                orientation = 'y') +
      scale_fill_manual(values = clrs) +
      # geom_line(aes(x = value,
      #               y = count,
      #               group = variable)) +
      # geom_point() +
      # scale_color_manual(values = clrs) +
      scale_y_reverse() + 
      labs(y = "Core no.",
           x= "Relative composition [%]",
           title = "Relative contribution of endmembers to the sediment composition")
      
    # x11()
    coreplt.file <- file.path(filebase,paste(nameing, "_coreplt.pdf",sep = ""))
    pdf(coreplt.file, height = 50, width = 10, useDingbats = FALSE)
    print(core.plt)
    graphics.off()
  }
} else {cat("\n", "The processing was limited to the first step.")}
  
##===================================
## to .RData and to excel
##-----------------------------------

## put a list of dfs to an excel
## ! input can only be a string; do not use actual variables !
## likewise, a list of strings
to.x <- function(ex){
  # nm <- deparse(substitute(ex))
  nm <- ex
  y.file <- file.path(filebase,paste0(nameing,"-",nm,".xlsx"))
  if(file.exists(y.file)){file.remove(y.file)}
  tmp <- eval(parse(text = ex))
  # print(tmp)
  if("list" %in% class(tmp)){
    for(y in seq_along(tmp)){
      if(exists("wb")){
        addWorksheet(wb, as.character(names(tmp[y])))
        writeDataTable(wb, as.character(names(tmp[y])), tmp[[y]])
      } else if(!exists("wb")){
        wb <- createWorkbook()
        addWorksheet(wb, as.character(names(tmp[y])))
        writeDataTable(wb, as.character(names(tmp[y])), tmp[[y]])
      }
    }
    ## out of loop saving the workbook
    saveWorkbook(wb, y.file)
    ##-----------------------------------
    # for(x in seq_along(tmp)){
    #   if(file.exists(x.file)){
    #     write.xlsx(tmp,
    #                x.file,
    #                sheetName = as.character(names(tmp[x])),
    #                append = TRUE)
    #   } else {
    #     write.xlsx(tmp,
    #                x.file,
    #                sheetName = as.character(names(tmp[x])))
    #   }
    # }
  } else if("data.table" %in% class(tmp) || "data.frame" %in% class(tmp)){
    write.xlsx(tmp,
               y.file)
  }
}

if(excel && exists("data.lst")){
  ## before anything is torn by the excel part
  ## better save the environment
  img <-  file.path(filebase, paste0(nameing, ".RData"))
  save.image(file = img); rm(img)
  
  for(e in data.lst){
    to.x(e) ## turn a string to variable: eval(parse(text = var))
            ## or: eval(as.name(var))
            ## opposite, turn variable name to string: deparse(substitute(var))
  }
}

rm(
  l
  ,m
  ,o
  ,a
  ,b
  ,p
  ,w
  ,e
  ,o.str
  ,rec.plot.tmp
  # ,all.dat.LD
  # ,all.dat.lda
  # ,all.dat.prd
  ,core.dat
  # ,core.dat.prd
  ,tmp
  ,tmp.cequ
  ,tmp.comp
  ,tmp.dat
  ,tmp.KW
  ,tmp.LDA
  ,tmp.pca
  ,tmp.pcons
  ,tmp.df
  ,tmp.pKW
  ,tmp.LDd
  ,tmp.psLDA
  ,tmp.stpwlda
  ,LD.plt
  ,tmp.MDS
  ,tmp.vis
  ,tmp.vis.cmp
  # ,tmp.umx
  ,tmp.plt
  # ,tmp.means
  ,cl.nms
  ,cntr
  ,compare.lda
  ,conservatio
  ,keeper
  ,tmp.cnt
  ,tmp.name
  ,tmp.ntr.nm
  ,tmp.stp
  ,clrs
  # ,img
  ,f1
  ,funck
  ,to.x
)

## before anything is torn by the excel part
## better save the environment
img <-  file.path(filebase, paste0(nameing, ".RData"))
save.image(file = img); rm(img)

##===================================
## Final
##-----------------------------------

## write the recorded graphics to a file
pdf(file = plt.file, height = 10, width = 10, useDingbats = FALSE)
for (my.plot in rec.plot.init) {
  replayPlot(my.plot)
}
graphics.off()
cat("\n","\n", "Apparently, we have made it through all the data you provided.",
    "\n", " You should now be able to find some plots at ", plt.file, ".",
    sep = "")

rm(rec.plot.init
   ,my.plot)

sink()
# dev.off(which = dev.cur())

cat("The End")
